@@ -14,6 +14,7 @@ module Msf
 module Exploit::Remote::HttpClient
 
   include Msf::Auxiliary::Report
+  include Msf::Auxiliary::LoginScanner
 
   #
   # Initializes an exploit module that exploits a vulnerability in an HTTP
@@ -107,7 +108,7 @@ module Exploit::Remote::HttpClient
   end
 
 
-  # This method is meant to be overriden in the exploit module to specify a set of regexps to
+  # This method is meant to be overridden in the exploit module to specify a set of regexps to
   # attempt to match against. A failure to match any of them results in a RuntimeError exception
   # being raised.
   #
@@ -152,7 +153,7 @@ module Exploit::Remote::HttpClient
     client_password = opts['password'] || datastore['HttpPassword'] || ''
 
     http_logger_subscriber = Rex::Proto::Http::HttpLoggerSubscriber.new(logger: self)
-    
+
     nclient = Rex::Proto::Http::Client.new(
       opts['rhost'] || rhost,
       (opts['rport'] || rport).to_i,
@@ -270,44 +271,39 @@ module Exploit::Remote::HttpClient
   # this method will override the defaults.
   #
   def configure_http_login_scanner(conf)
-    {
-      host:                          rhost,
-      port:                          rport,
-      ssl:                           ssl,
-      ssl_version:                   ssl_version,
-      proxies:                       datastore['PROXIES'],
-      framework:                     framework,
-      framework_module:              self,
-      vhost:                         vhost,
-      user_agent:                    datastore['UserAgent'],
-      evade_uri_encode_mode:         datastore['HTTP::uri_encode_mode'],
-      evade_uri_full_url:            datastore['HTTP::uri_full_url'],
-      evade_pad_method_uri_count:    datastore['HTTP::pad_method_uri_count'],
-      evade_pad_uri_version_count:   datastore['HTTP::pad_uri_version_count'],
-      evade_pad_method_uri_type:     datastore['HTTP::pad_method_uri_type'],
-      evade_pad_uri_version_type:    datastore['HTTP::pad_uri_version_type'],
-      evade_method_random_valid:     datastore['HTTP::method_random_valid'],
-      evade_method_random_invalid:   datastore['HTTP::method_random_invalid'],
-      evade_method_random_case:      datastore['HTTP::method_random_case'],
-      evade_version_random_valid:    datastore['HTTP::version_random_valid'],
-      evade_version_random_invalid:  datastore['HTTP::version_random_invalid'],
-      evade_uri_dir_self_reference:  datastore['HTTP::uri_dir_self_reference'],
-      evade_uri_dir_fake_relative:   datastore['HTTP::uri_dir_fake_relative'],
-      evade_uri_use_backslashes:     datastore['HTTP::uri_use_backslashes'],
-      evade_pad_fake_headers:        datastore['HTTP::pad_fake_headers'],
-      evade_pad_fake_headers_count:  datastore['HTTP::pad_fake_headers_count'],
-      evade_pad_get_params:          datastore['HTTP::pad_get_params'],
-      evade_pad_get_params_count:    datastore['HTTP::pad_get_params_count'],
-      evade_pad_post_params:         datastore['HTTP::pad_post_params'],
-      evade_pad_post_params_count:   datastore['HTTP::pad_post_params_count'],
-      evade_shuffle_get_params:      datastore['HTTP::shuffle_get_params'],
-      evade_shuffle_post_params:     datastore['HTTP::shuffle_post_params'],
-      evade_uri_fake_end:            datastore['HTTP::uri_fake_end'],
-      evade_uri_fake_params_start:   datastore['HTTP::uri_fake_params_start'],
-      evade_header_folding:          datastore['HTTP::header_folding'],
-      ntlm_domain:                   datastore['DOMAIN'],
-      digest_auth_iis:               datastore['DigestAuthIIS']
-    }.merge(conf)
+    configure_login_scanner(
+      {
+        vhost:                         vhost,
+        user_agent:                    datastore['UserAgent'],
+        evade_uri_encode_mode:         datastore['HTTP::uri_encode_mode'],
+        evade_uri_full_url:            datastore['HTTP::uri_full_url'],
+        evade_pad_method_uri_count:    datastore['HTTP::pad_method_uri_count'],
+        evade_pad_uri_version_count:   datastore['HTTP::pad_uri_version_count'],
+        evade_pad_method_uri_type:     datastore['HTTP::pad_method_uri_type'],
+        evade_pad_uri_version_type:    datastore['HTTP::pad_uri_version_type'],
+        evade_method_random_valid:     datastore['HTTP::method_random_valid'],
+        evade_method_random_invalid:   datastore['HTTP::method_random_invalid'],
+        evade_method_random_case:      datastore['HTTP::method_random_case'],
+        evade_version_random_valid:    datastore['HTTP::version_random_valid'],
+        evade_version_random_invalid:  datastore['HTTP::version_random_invalid'],
+        evade_uri_dir_self_reference:  datastore['HTTP::uri_dir_self_reference'],
+        evade_uri_dir_fake_relative:   datastore['HTTP::uri_dir_fake_relative'],
+        evade_uri_use_backslashes:     datastore['HTTP::uri_use_backslashes'],
+        evade_pad_fake_headers:        datastore['HTTP::pad_fake_headers'],
+        evade_pad_fake_headers_count:  datastore['HTTP::pad_fake_headers_count'],
+        evade_pad_get_params:          datastore['HTTP::pad_get_params'],
+        evade_pad_get_params_count:    datastore['HTTP::pad_get_params_count'],
+        evade_pad_post_params:         datastore['HTTP::pad_post_params'],
+        evade_pad_post_params_count:   datastore['HTTP::pad_post_params_count'],
+        evade_shuffle_get_params:      datastore['HTTP::shuffle_get_params'],
+        evade_shuffle_post_params:     datastore['HTTP::shuffle_post_params'],
+        evade_uri_fake_end:            datastore['HTTP::uri_fake_end'],
+        evade_uri_fake_params_start:   datastore['HTTP::uri_fake_params_start'],
+        evade_header_folding:          datastore['HTTP::header_folding'],
+        ntlm_domain:                   datastore['DOMAIN'],
+        digest_auth_iis:               datastore['DigestAuthIIS'],
+      }.merge(conf)
+    )
   end
 
   #
