@@ -272,16 +272,17 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
     auth_context
   end
 
-  def get_message_encryptor(key, client_sequence_number, server_sequence_number)
+  def get_message_encryptor(key, client_sequence_number, server_sequence_number, use_acceptor_subkey: true, rc4_pad_style: :single_byte)
     Rex::Proto::Gss::Kerberos::MessageEncryptor.new(key,
                                             client_sequence_number,
                                             server_sequence_number,
                                             is_initiator: true,
-                                            use_acceptor_subkey: true,
-                                            dce_style: @dce_style)
+                                            use_acceptor_subkey: use_acceptor_subkey,
+                                            dce_style: @dce_style,
+                                            rc4_pad_style: rc4_pad_style)
   end
 
-  def parse_gss_init_response(token, session_key, mechanism: 'kerberos')
+  def parse_gss_init_response(token, session_key)
     mech_id, encapsulated_token = unwrap_pseudo_asn1(token)
 
     if mech_id.value == Rex::Proto::Gss::OID_KERBEROS_5.value
@@ -640,7 +641,7 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
   #
   # @param [Rex::Proto::Kerberos::CredentialCache::Krb5CcacheCredential] credential
   # @param [Hash] _options
-  def authenticate_via_krb5_ccache_credential_tgs(credential, _options = {})
+  def authenticate_via_krb5_ccache_credential_tgs(credential, options = {})
     unless credential.is_a?(Rex::Proto::Kerberos::CredentialCache::Krb5CcacheCredential)
       raise TypeError, 'credential must be a Krb5CcacheCredential instance'
     end
@@ -666,7 +667,7 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
 
     ## Service Authentication
     checksum = nil
-    checksum = build_gss_ap_req_checksum_value(mutual_auth, dce_style, nil, nil, nil, nil, nil) if use_gss_checksum
+    checksum = build_gss_ap_req_checksum_value(options: options) if use_gss_checksum
 
     sequence_number = rand(1 << 32)
     service_ap_request = build_service_ap_request(
@@ -755,13 +756,10 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
     checksum = nil
     if use_gss_checksum
       checksum = build_gss_ap_req_checksum_value(
-        mutual_auth,
-        dce_style,
-        delegated_tgs_ticket,
-        delegated_tgs_auth,
-        tgs_auth.key,
-        realm,
-        client_name
+        ticket: delegated_tgs_ticket,
+        decrypted_part: delegated_tgs_auth,
+        session_key: tgs_auth.key,
+        options: options
       )
     end
 
@@ -791,15 +789,21 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
     }
   end
 
-  def build_gss_ap_req_checksum_value(mutual_auth, dce_style, ticket, decrypted_part, session_key, realm, client_name)
+  def build_gss_ap_req_checksum_value(ticket: nil, decrypted_part: nil, session_key: nil, options: {})
     # @see https://datatracker.ietf.org/doc/html/rfc4121#section-4.1.1
-    # No channel binding
-    channel_binding_info = "\x00" * 16
+
+    if options[:gss_channel_binding]
+      channel_binding_info = options[:gss_channel_binding].channel_binding_token
+    else
+      channel_binding_info = "\x00".b * 16
+    end
     channel_binding_info_len = [channel_binding_info.length].pack('V')
 
-    flags = GSS_REPLAY_DETECT | GSS_SEQUENCE | GSS_CONFIDENTIAL | GSS_INTEGRITY
-    flags |= GSS_MUTUAL if mutual_auth
-    flags |= GSS_DCE_STYLE if dce_style
+    flags = GSS_REPLAY_DETECT | GSS_SEQUENCE
+    flags |= GSS_CONFIDENTIAL if options.fetch(:gss_flag_confidential, true)
+    flags |= GSS_INTEGRITY if options.fetch(:gss_flag_integrity, true)
+    flags |= GSS_MUTUAL if options.fetch(:gss_flag_mutual) { mutual_auth }
+    flags |= GSS_DCE_STYLE if options.fetch(:gss_flag_dce_style) { dce_style }
     flags |= GSS_DELEGATE if ticket
 
     flags = [flags].pack('V')
@@ -813,8 +817,8 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
       krb_cred.tickets = [ticket]
       ticket_info = Rex::Proto::Kerberos::Model::KrbCredInfo.new
       ticket_info.key = decrypted_part.key
-      ticket_info.prealm = realm
-      ticket_info.pname = build_client_name(client_name: client_name)
+      ticket_info.prealm = options.fetch(:realm) { self.realm.upcase }
+      ticket_info.pname = build_client_name(client_name: options.fetch(:client_name) { username })
       ticket_info.flags = decrypted_part.flags
       ticket_info.auth_time = decrypted_part.auth_time
       ticket_info.start_time = decrypted_part.start_time
