@@ -333,6 +333,10 @@ module Msf
       false
     end
 
+    def default_options
+      self.module_info['DefaultOptions']
+    end
+
     def required_cred_options
       @required_cred_options ||= lambda {
         self.options.select { |name, opt|
@@ -455,6 +459,8 @@ module Msf
     attr_writer   :privileged # :nodoc:
     attr_writer   :license # :nodoc:
 
+
+
   end
 
 end
