@@ -74,6 +74,104 @@ class Net::LDAP::Connection # :nodoc:
     end
   end
 
+  # Allow wrapping the socket to read and write SASL data
+  module SocketSaslIO
+    include Rex::Proto::Sasl
+
+    # This seems hacky, but we're just fitting in with how net-ldap does it
+    def get_ber_length(data)
+      n = data[0].ord
+
+      if n <= 0x7f
+        [n, 1]
+      elsif n == 0x80
+        raise Net::BER::BerError,
+            'Indeterminite BER content length not implemented.'
+      elsif n == 0xff
+        raise Net::BER::BerError, 'Invalid BER length 0xFF detected.'
+      else
+        v = 0
+        extra_length = n & 0x7f
+        data[1,n & 0x7f].each_byte do |b|
+          v = (v << 8) + b
+        end
+
+        [v, extra_length + 1]
+      end
+    end
+
+    def read_ber(syntax = nil)
+      unless @wrap_read.nil?
+        if ber_cache.any?
+          return ber_cache.shift
+        end
+        # SASL buffer length
+        length_bytes = read(4)
+        # The implementation in net-ldap returns nil if it doesn't read any data
+        return nil unless length_bytes
+
+        length = length_bytes.unpack('N')[0]
+
+        # Now read the actual data
+        data = read(length)
+
+        # Decrypt it
+        plaintext = @wrap_read.call(data)
+
+        while plaintext.length > 0
+          id = plaintext[0].ord
+          ber_length, used_chars = get_ber_length(plaintext[1,plaintext.length])
+          plaintext = plaintext[1+used_chars, plaintext.length]
+
+          # We may receive several objects in the one packet
+          # Ideally we'd refactor all of ruby-net-ldap to use
+          # yields for this, but it's all a bit messy. So instead,
+          # just store them all and return the next one each time
+          # we're asked.
+          ber_cache.append(parse_ber_object(syntax, id, plaintext[0,ber_length]))
+
+          plaintext = plaintext[ber_length,plaintext.length]
+        end
+
+        return ber_cache.shift
+      else
+        super(syntax)
+      end
+    end
+
+    def write(data)
+      unless @wrap_write.nil?
+        # Encrypt it
+        data = @wrap_write.call(data)
+
+        # Prepend the length bytes
+        data = wrap_sasl(data)
+      end
+
+      super(data)
+    end
+
+    def setup(wrap_read, wrap_write)
+      @wrap_read = wrap_read
+      @wrap_write = wrap_write
+      @ber_cache = []
+    end
+
+    private
+
+    attr_accessor :wrap_read
+    attr_accessor :wrap_write
+    attr_accessor :ber_cache
+  end
+
+  module ConnectionSaslIO
+    # Provide the encryption wrapper for the caller to set up
+    def wrap_read_write(wrap_read, wrap_write)
+      @conn.extend(SocketSaslIO)
+      @conn.setup(wrap_read, wrap_write)
+    end
+  end
+
   # Initialize the LDAP connection using Rex::Socket::TCP,
   # and optionally set up encryption on the connection if configured.
   #
@@ -87,9 +185,13 @@ class Net::LDAP::Connection # :nodoc:
       @conn = Rex::Socket::Tcp.create(
         'PeerHost' => server[:host],
         'PeerPort' => server[:port],
-        'Proxies' => server[:proxies]
+        'Proxies' => server[:proxies],
+        'Timeout' => server[:connect_timeout]
       )
       @conn.extend(SynchronousRead)
+
+      # Set up read/write wrapping
+      self.extend(ConnectionSaslIO)
     rescue SocketError
       raise Net::LDAP::LdapError, 'No such address or other socket error.'
     rescue Errno::ECONNREFUSED
